module Kakimasu
  class KeysController < ::ApplicationController
    # Include neccessary helpers
    include SearchTranslationsHelper
    include GroupingHelper
    include SearchKeyHelper
    include PaginationHelper

    before_action :categories
    before_action :set_keys_category

    def index
      # Check if user has permissions
      authorize :translation, :index?
      policy_scope(Translation)

      set_page_size(5)                              # set number of keys in one page
      params[:page] = 1 unless params[:page]        # if page number is not stated, then by default it is 1. page
      @keys = search_for_translations(@category)    # search for used keys
      @active_tab = 'keys'                          # sets active tab in navigation
    end

    def create
      # Authorize user
      authorize :translation, :create?

      before_key = params[:key]     # Varibale contains key before create method was executed

      # Check if path to translation key is given
      enable_lazy_lookup unless params[:path].blank?

      # Variable which is true when function saves changes of something.
      changed = false

      # Save entered translations
      I18n.available_locales.each do |l|
        params[l].gsub!("'", "&#39;")     # Substitutes quotes with quotes HTML code.

        # If there is translation which is not equal with old one, then it is saved
        unless params[l].blank? || I18n.t(params[:key], locale: l) == params[l]
          changed = true if (I18n.backend.store_translations(l, { params[:key] => params[l] }, escape: false))
        end
      end
      I18n.backend.store_translations(params[:language], { params[:key] => params[:value] }, escape: false) if params[:language]

      # Process ajax process diferently
      if request.xhr?
        respond_to do |format|
          format.js {}
        end
      else
        redirect_to kakimasu_keys_path(keys_category: @category)
      end

      # If the key is changed then it means there were changes
      changed = true if before_key != params[:key]

      # Shows result to user
      if changed
        flash.notice = t('translation_succesfully_stored')
      else
        flash.alert = t('nothing_was_changed')
      end
    end

    private

    # All available categories in "key management" view
    def categories
      @categories = ['missing_translations', 'all_translation_keys', 'active_record_attribute_translations', 'active_record_model_translations']
    end

    # Set current category
    def set_keys_category
      @category = params[:keys_category] || 'missing_translations'
    end

    # Enable/disable 'lazy lookup' on key based on checkbox 'unique' value
    def enable_lazy_lookup
      if params[:lazy_lookup]
        lazy_lookup_key(params[:key], params[:path])
        params[:key] = prefix(params[:path]) + '.' + params[:key] unless params[:key].start_with?(prefix(params[:path]))
      else
        remove_lazy_lookup(params[:key], params[:path])
        params[:key] = params[:key].split('.').last
      end
    end
  end
end