module SearchTranslationsHelper
  # Method to get prefix for lazy lookup
  def prefix(path)
    prefix = path.split('/')
    prefix[-1] = prefix.last.split('.').first
    prefix = prefix.join('.')
  end

  # Method returns if key has missing translation in one of locales
  def missing_translations?(key)
    missing_translation = false
    I18n.available_locales.each do |locale|
      if !I18n.exists?(key, locale)
        missing_translation = true  # If in one of locales key has no translation, method returns true
      end
    end
    missing_translation
  end

  # Method searches used translations keys in a file
  def find_keys(file, path)
    array = file.scan(/[=|\s][t][\s]*[(]?[\s]*['|:]([.]*\w*)/).uniq
    keys = Array.new

    # If key has '.' in front of it, it should be completed as lazy lookup key.
    array.each_with_index do |key, index|
      if key.first[0] == '.'
        array[index] = key.first.insert(0, prefix(path))
      end
    end
    array
  end

  # Method returns array of keys with missing translations from given array
  def find_missing_translations_keys(array)
    missing_translations_keys = []

    array.each do |key|
      missing_translations_keys.push key if missing_translations?(key)
    end

    missing_translations_keys
  end

  # Method searches used active record human attribute name keys in given file
  def find_attribute_keys(file)
    array = file.scan(/(\w*).human_attribute_name[(][:](\w*)/).uniq
    keys = []
    # Keys are transformed into correct active record attribute keys form
    array.each do |arr|
      arr = 'activerecord.' + 'attributes.' + arr.first.downcase + '.' + arr.last
      keys.push arr
    end
    keys
  end

  # Method searches used Model name keys in given file
  def find_model_name_keys(file)
    array = file.scan(/(\w*).model_name.human/).uniq
    keys = []
    # Keys are transformed into correct active record model name keys form
    array.each do |arr|
      arr = 'activerecord.models.' + arr.first.downcase
      keys.push arr
    end
    keys
  end

  # Method returns array with requested translation keys
  def search_for_translations(category)
    keys = []

    Dir.glob("app/views/**/*").each do |path|
      if File.file?(path)
        file = File.read(path)
        case category
        when 'active_record_attribute_translations'
          keys.push find_attribute_keys(file)
        when 'active_record_model_translations'
          keys.push find_model_name_keys(file)
        when 'all_translation_keys'
          path.slice! "app/views/"
          keys.push find_keys(file, path).push path
        when 'missing_translations'
          path.slice! "app/views/"
          # push in array only those keys which have missing translations
          keys.push find_missing_translations_keys(find_keys(file, path)).push path
        end
      end
    end

    case category
    when 'active_record_attribute_translations'
      keys.flatten.uniq
    when 'active_record_model_translations'
      keys.flatten.uniq
    else
      keys
    end
  end
end
