I18n.class_eval do
  def self.translate(*args)
    options  = args.last.is_a?(Hash) ? args.pop.dup : {}
    key      = args.shift
    backend  = config.backend
    locale   = options.delete(:locale) || config.locale
    handling = options.delete(:throw) && :throw || options.delete(:raise) && :raise
    enforce_available_locales!(locale)
    raise I18n::ArgumentError if key.is_a?(String) && key.empty?

    result = catch(:exception) do
      if key.is_a?(Array)
        key.map { |k| backend.translate(locale, k, options) }
      else
        backend.translate(locale, key, options)
      end
    end

    # For sending available translations to javascript
    all_translations = []
    if Thread.current[:show_translation_key] == true
      # For array is needed just translations without <span> tags as it would be if show_translation_key would be false
      Thread.current[:show_translation_key] = false
      I18n.available_locales.each do |l|
        translation = catch(:exception) do
          if key.is_a?(Array)
            all_translations.push(key.map { |k| backend.translate(l, k, options) })
          else
            all_translations.push(backend.translate(l, key, options))
          end
        end
        all_translations.push('') if translation.is_a?(I18n::MissingTranslation)
      end
      Thread.current[:show_translation_key] = true
    end

    if result.is_a?(I18n::MissingTranslation)
      if Thread.current[:show_translation_key] == true && !Array.wrap(options[:scope]).flatten.join('.').include?('simple_form')
        "<span class='translation_missing translation-with-key' data-component='TranslationPopover' data-content='#{key}'"\
        "data-locales='#{I18n.available_locales}' data-translations='#{all_translations}'"\
        "data-unique=#{(key.to_s.include?('.') && key.to_s.split('.').first != 'activerecord')}>#{key}</span>".html_safe
      else
        handle_exception(handling, result, locale, key, options)
      end
    elsif result.is_a?(String) && Thread.current[:show_translation_key] && (result.present? || !Array.wrap(options[:scope]).flatten.join('.').include?('simple_form'))
      # Monkey patching for getting key and putting into spans
      s = options[:scope] if options[:scope]
      d = Array.wrap(options[:default]).first if options[:default]
      k = d if key.to_s.blank?
      k = [s, key].flatten.compact.join('.')
      "<span class='translation-with-key' data-component='TranslationPopover' data-content='#{k}'"\
      "data-locales='#{I18n.available_locales}' data-translations='#{all_translations}'"\
      "data-unique=#{(k.to_s.include?('.') && k.to_s.split('.').first != 'activerecord')}>#{result}</span>".html_safe
    else
      result.to_s.html_safe
    end
  end
end
