module Kakimasu
  module Generators
    class BackupGenerator < Rails::Generators::Base
      # Groups all added keys from TRANSLATION_STORE
      grouped_keys_by_locale = TRANSLATION_STORE.keys.group_by { |el| el.partition('.').first }

      # Checks if there is at least one translation in TRANSLATION_STORE
      if TRANSLATION_STORE.keys.count == 0
        puts '**************************************************'
        puts 'Backup can not be done. There are no translations!'
        puts '**************************************************'
      else

        backup_file = '.kakimasu_backup.yml'

        # Check if backup file already exists
        unless File.file?(backup_file)
          puts 'Creating backup_file...'

          # If there is no backup file then it is created here
          File.write(backup_file, "")
        end

        # Read Kakimasu backup file
        file_content = File.read(backup_file)

        # Modify file content - write all translations in there
        I18n.available_locales.each do |locale|
          position = 0      # Position in string where to start write
          exists = false    # Tells if this locale is saved in backup (if not then after all translations will be writed 'new line character' to seperate languages)

          # Search where to start insert new translations in string (Searching goes by lines)
          file_content.each_line do |line|
            position += line.length

            # If locale is found then break the loop
            if line.start_with?("#{locale}:")
              exists = true
              break
            end
          end

          # If locale doesn't exist in the file already then start writing from file start and insert new line just to seperate languages
          unless exists
            position = 0
            file_content.insert(0, "\n")
          end

          # If there is at least one translation in current language just then will insert new translations
          unless grouped_keys_by_locale[locale.to_s].nil?
            grouped_keys_by_locale[locale.to_s].each do |key|
              splitted_key = key.split('.')

              # Case if translation is not unique (it can be normally saved without special indentation requirements)
              if splitted_key.count == 2
                translation_key = key.partition('.').last
                translation = TRANSLATION_STORE[key]

                file_content.insert(position, "  #{translation_key}: #{translation}\n")

              # Case if translation is unique (Special indentation requirements)
              elsif splitted_key.count > 2
                unique_translation = ''     # Variable for unique translation with indentation

                splitted_key.each_with_index do |subkey, index|
                  unless index == 0
                    indentation = '  ' * index
                    translation = TRANSLATION_STORE[key]

                    # Last 'subkey' after colon has translation
                    if index == splitted_key.count - 1
                      unique_translation += "#{indentation}#{subkey}: #{translation}\n"
                    else
                      unique_translation += "#{indentation}#{subkey}:\n"
                    end
                  end
                end

                file_content.insert(position, unique_translation)
              end
            end
          end

          # Insert locale in front of all its translations if it wasnt already there
          file_content.insert(0, "#{locale}:\n") unless exists
        end

        # Open Kakimasu backup file and write new content
        File.open(backup_file, 'w+') do |f|
          f.write(file_content)
        end

        # Success message
        puts ''
        puts "*********************************************************"
        puts "Translations successfully stored in #{backup_file}!"
        puts "*********************************************************"
      end
    end
  end
end